@@ -1,9 +1,12 @@
-use crate::path;
+use crate::{io, path};
 use bitflags::bitflags;
 use io_lifetimes::OwnedFd;
-use std::{ffi::CStr, io};
+use std::ffi::CStr;
 #[cfg(libc)]
-use {crate::negone_err, std::convert::TryInto, unsafe_io::os::posish::FromRawFd};
+use {
+    crate::negone_err,
+    unsafe_io::os::posish::{FromRawFd, RawFd},
+};
 
 #[cfg(libc)]
 bitflags! {
@@ -32,7 +35,7 @@ bitflags! {
 /// `memfd_create(path, flags)`
 #[inline]
 pub fn memfd_create<P: path::Arg>(path: P, flags: MemfdFlags) -> io::Result<OwnedFd> {
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _memfd_create(&path, flags)
 }
 
@@ -46,7 +49,7 @@ fn _memfd_create(path: &CStr, flags: MemfdFlags) -> io::Result<OwnedFd> {
         ))?;
 
         #[allow(clippy::useless_conversion)]
-        Ok(OwnedFd::from_raw_fd(fd.try_into().unwrap()))
+        Ok(OwnedFd::from_raw_fd(fd as RawFd))
     }
 }
 
