@@ -1,7 +1,6 @@
 #[cfg(not(target_os = "wasi"))]
-use crate::io::Termios;
+use crate::io::{self, Termios, Winsize};
 use io_lifetimes::{AsFd, BorrowedFd};
-use std::io;
 #[cfg(libc)]
 use {crate::zero_ok, std::mem::MaybeUninit, unsafe_io::os::posish::AsRawFd};
 
@@ -60,7 +59,7 @@ fn _ioctl_tcgets(fd: BorrowedFd<'_>) -> io::Result<Termios> {
     crate::linux_raw::ioctl_tcgets(fd)
 }
 
-/// Also known as `fcntl(fildes, F_SETFD, FD_CLOEXEC)`.
+/// Also known as `fcntl(fd, F_SETFD, FD_CLOEXEC)`.
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 #[inline]
 pub fn ioctl_fioclex<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<()> {
@@ -72,3 +71,30 @@ pub fn ioctl_fioclex<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<()> {
 fn _ioctl_fioclex(fd: BorrowedFd<'_>) -> io::Result<()> {
     unsafe { zero_ok(libc::ioctl(fd.as_raw_fd(), libc::FIOCLEX)) }
 }
+
+/// `ioctl(fd, TIOCGWINSZ)`.
+#[cfg(not(target_os = "wasi"))]
+#[inline]
+pub fn ioctl_tiocgwinsz(fd: BorrowedFd) -> io::Result<Winsize> {
+    let fd = fd.as_fd();
+    _ioctl_tiocgwinsz(fd)
+}
+
+#[cfg(all(libc, not(target_os = "wasi")))]
+fn _ioctl_tiocgwinsz(fd: BorrowedFd) -> io::Result<Winsize> {
+    unsafe {
+        let mut buf = MaybeUninit::uninit();
+        zero_ok(libc::ioctl(
+            fd.as_raw_fd(),
+            libc::TIOCGWINSZ.into(),
+            buf.as_mut_ptr(),
+        ))?;
+        Ok(buf.assume_init())
+    }
+}
+
+#[cfg(linux_raw)]
+#[inline]
+fn _ioctl_tiocgwinsz(fd: BorrowedFd) -> io::Result<Winsize> {
+    crate::linux_raw::ioctl_tiocgwinsz(fd)
+}
