@@ -1,14 +1,16 @@
 use crate::{
-    io,
+    as_ptr, io,
     net::{SocketAddr, SocketAddrUnix, SocketAddrV4, SocketAddrV6},
 };
-#[cfg(any(linux_raw, all(libc, not(any(target_os = "ios", target_os = "macos")))))]
 use bitflags::bitflags;
 use io_lifetimes::{AsFd, BorrowedFd, OwnedFd};
-use std::mem::{size_of, MaybeUninit};
 use std::os::raw::c_int;
 #[cfg(linux_raw)]
 use std::os::raw::c_uint;
+use std::{
+    mem::{size_of, MaybeUninit},
+    ptr,
+};
 #[cfg(libc)]
 use {
     crate::{negone_err, zero_ok},
@@ -67,6 +69,7 @@ pub enum SocketType {
 #[non_exhaustive]
 pub enum AddressFamily {
     /// `AF_LOCAL`, aka `AF_UNIX`
+    #[doc(alias = "Unix")]
     Local = libc::AF_LOCAL as u32,
 
     /// `AF_INET`
@@ -92,6 +95,7 @@ pub enum AddressFamily {
 #[non_exhaustive]
 pub enum AddressFamily {
     /// `AF_LOCAL`, aka `AF_UNIX`
+    #[doc(alias = "Unix")]
     Local = linux_raw_sys::general::AF_LOCAL,
 
     /// `AF_INET`
@@ -277,13 +281,16 @@ pub enum Shutdown {
     ReadWrite = linux_raw_sys::general::SHUT_RDWR,
 }
 
-#[cfg(all(libc, not(any(target_os = "ios", target_os = "macos"))))]
+#[cfg(libc)]
 bitflags! {
     /// `SOCK_*` constants for `accept`.
     pub struct AcceptFlags: c_int {
         /// `SOCK_NONBLOCK`
+        #[cfg(not(any(target_os = "ios", target_os = "macos")))]
         const NONBLOCK = libc::SOCK_NONBLOCK;
+
         /// `SOCK_CLOEXEC`
+        #[cfg(not(any(target_os = "ios", target_os = "macos")))]
         const CLOEXEC = libc::SOCK_CLOEXEC;
     }
 }
@@ -334,7 +341,7 @@ fn _bind_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
     unsafe {
         zero_ok(libc::bind(
             sockfd.as_raw_fd(),
-            addr as *const _ as *const _,
+            as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrUnix>() as socklen_t,
         ))
     }
@@ -357,7 +364,7 @@ fn _bind_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
     unsafe {
         zero_ok(libc::bind(
             sockfd.as_raw_fd(),
-            addr as *const _ as *const _,
+            as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrV4>() as socklen_t,
         ))
     }
@@ -380,7 +387,7 @@ fn _bind_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
     unsafe {
         zero_ok(libc::bind(
             sockfd.as_raw_fd(),
-            addr as *const _ as *const _,
+            as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrV6>() as socklen_t,
         ))
     }
@@ -403,7 +410,7 @@ fn _connect_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()>
     unsafe {
         zero_ok(libc::connect(
             sockfd.as_raw_fd(),
-            addr as *const _ as *const _,
+            as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrUnix>() as socklen_t,
         ))
     }
@@ -426,7 +433,7 @@ fn _connect_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
     unsafe {
         zero_ok(libc::connect(
             sockfd.as_raw_fd(),
-            addr as *const _ as *const _,
+            as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrV4>() as socklen_t,
         ))
     }
@@ -449,7 +456,7 @@ fn _connect_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
     unsafe {
         zero_ok(libc::connect(
             sockfd.as_raw_fd(),
-            addr as *const _ as *const _,
+            as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrV6>() as socklen_t,
         ))
     }
@@ -478,64 +485,25 @@ fn _listen(sockfd: BorrowedFd<'_>, backlog: c_int) -> io::Result<()> {
     crate::linux_raw::listen(sockfd, backlog)
 }
 
-/// `accept(fd, addr, len)`
-#[cfg(any(target_os = "ios", target_os = "macos"))]
-#[inline]
-pub fn accept<'f, Fd: AsFd<'f>>(sockfd: Fd) -> io::Result<(OwnedFd, SocketAddr)> {
-    let sockfd = sockfd.as_fd();
-    _accept(sockfd)
-}
-
-#[cfg(all(libc, any(target_os = "ios", target_os = "macos")))]
-fn _accept(sockfd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketAddr)> {
-    unsafe {
-        let mut storage = MaybeUninit::<sockaddr_storage>::uninit();
-        let mut len = size_of::<sockaddr_storage>() as socklen_t;
-        let raw_fd = negone_err(libc::accept(
-            sockfd.as_raw_fd(),
-            storage.as_mut_ptr() as *mut _,
-            &mut len,
-        ))?;
-        let owned_fd = OwnedFd::from_raw_fd(raw_fd);
-        let storage = storage.assume_init();
-        let addr = match i32::from(storage.ss_family) {
-            libc::AF_INET => {
-                assert!(len as usize >= size_of::<SocketAddrV4>());
-                SocketAddr::V4((*(&storage as *const _ as *const SocketAddrV4)).clone())
-            }
-            libc::AF_INET6 => {
-                assert!(len as usize >= size_of::<SocketAddrV6>());
-                SocketAddr::V6((*(&storage as *const _ as *const SocketAddrV6)).clone())
-            }
-            libc::AF_LOCAL => {
-                assert!(len as usize >= size_of::<SocketAddrUnix>());
-                SocketAddr::Unix((*(&storage as *const _ as *const SocketAddrUnix)).clone())
-            }
-            _ => panic!(),
-        };
-        Ok((owned_fd, addr))
-    }
-}
-
 /// `accept4(fd, addr, len, flags)`
-#[cfg(not(any(target_os = "ios", target_os = "macos")))]
 #[inline]
-pub fn accept4<'f, Fd: AsFd<'f>>(
+#[doc(alias = "accept4")]
+pub fn accept<'f, Fd: AsFd<'f>>(
     sockfd: Fd,
     flags: AcceptFlags,
 ) -> io::Result<(OwnedFd, SocketAddr)> {
     let sockfd = sockfd.as_fd();
-    _accept4(sockfd, flags)
+    _accept(sockfd, flags)
 }
 
 #[cfg(all(libc, not(any(target_os = "ios", target_os = "macos"))))]
-fn _accept4(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd, SocketAddr)> {
+fn _accept(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd, SocketAddr)> {
     unsafe {
         let mut storage = MaybeUninit::<sockaddr_storage>::uninit();
         let mut len = size_of::<sockaddr_storage>() as socklen_t;
         let raw_fd = negone_err(libc::accept4(
             sockfd.as_raw_fd(),
-            storage.as_mut_ptr() as *mut _,
+            storage.as_mut_ptr().cast::<_>(),
             &mut len,
             flags.bits(),
         ))?;
@@ -544,15 +512,48 @@ fn _accept4(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd,
         let addr = match i32::from(storage.ss_family) {
             libc::AF_INET => {
                 assert!(len as usize >= size_of::<SocketAddrV4>());
-                SocketAddr::V4((*(&storage as *const _ as *const SocketAddrV4)).clone())
+                SocketAddr::V4(ptr::read(as_ptr(&storage).cast::<_>()))
             }
             libc::AF_INET6 => {
                 assert!(len as usize >= size_of::<SocketAddrV6>());
-                SocketAddr::V6((*(&storage as *const _ as *const SocketAddrV6)).clone())
+                SocketAddr::V6(ptr::read(as_ptr(&storage).cast::<_>()))
             }
             libc::AF_LOCAL => {
                 assert!(len as usize >= size_of::<SocketAddrUnix>());
-                SocketAddr::Unix((*(&storage as *const _ as *const SocketAddrUnix)).clone())
+                SocketAddr::Unix(ptr::read(as_ptr(&storage).cast::<_>()))
+            }
+            _ => panic!(),
+        };
+        Ok((owned_fd, addr))
+    }
+}
+
+/// Darwin lacks `accept4`, but does have `accept`. We define
+/// `AcceptFlags` to have no flags, so we can discard it here.
+#[cfg(all(libc, any(target_os = "ios", target_os = "macos")))]
+fn _accept(sockfd: BorrowedFd<'_>, _flags: AcceptFlags) -> io::Result<(OwnedFd, SocketAddr)> {
+    unsafe {
+        let mut storage = MaybeUninit::<sockaddr_storage>::uninit();
+        let mut len = size_of::<sockaddr_storage>() as socklen_t;
+        let raw_fd = negone_err(libc::accept(
+            sockfd.as_raw_fd(),
+            storage.as_mut_ptr().cast::<_>(),
+            &mut len,
+        ))?;
+        let owned_fd = OwnedFd::from_raw_fd(raw_fd);
+        let storage = storage.assume_init();
+        let addr = match i32::from(storage.ss_family) {
+            libc::AF_INET => {
+                assert!(len as usize >= size_of::<SocketAddrV4>());
+                SocketAddr::V4(ptr::read(as_ptr(&storage).cast::<_>()))
+            }
+            libc::AF_INET6 => {
+                assert!(len as usize >= size_of::<SocketAddrV6>());
+                SocketAddr::V6(ptr::read(as_ptr(&storage).cast::<_>()))
+            }
+            libc::AF_LOCAL => {
+                assert!(len as usize >= size_of::<SocketAddrUnix>());
+                SocketAddr::Unix(ptr::read(as_ptr(&storage).cast::<_>()))
             }
             _ => panic!(),
         };
@@ -562,21 +563,21 @@ fn _accept4(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd,
 
 #[cfg(linux_raw)]
 #[inline]
-fn _accept4(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd, SocketAddr)> {
-    let (owned_fd, storage, len) = crate::linux_raw::accept4(sockfd, flags.bits())?;
+fn _accept(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd, SocketAddr)> {
+    let (owned_fd, storage, len) = crate::linux_raw::accept(sockfd, flags.bits())?;
     let addr = unsafe {
         match u32::from(storage.ss_family) {
             linux_raw_sys::general::AF_INET => {
                 assert!(len as usize >= size_of::<SocketAddrV4>());
-                SocketAddr::V4((*(&storage as *const _ as *const SocketAddrV4)).clone())
+                SocketAddr::V4(ptr::read(as_ptr(&storage).cast::<_>()))
             }
             linux_raw_sys::general::AF_INET6 => {
                 assert!(len as usize >= size_of::<SocketAddrV6>());
-                SocketAddr::V6((*(&storage as *const _ as *const SocketAddrV6)).clone())
+                SocketAddr::V6(ptr::read(as_ptr(&storage).cast::<_>()))
             }
             linux_raw_sys::general::AF_LOCAL => {
                 assert!(len as usize >= size_of::<SocketAddrUnix>());
-                SocketAddr::Unix((*(&storage as *const _ as *const SocketAddrUnix)).clone())
+                SocketAddr::Unix(ptr::read(as_ptr(&storage).cast::<_>()))
             }
             _ => panic!(),
         }
@@ -635,8 +636,10 @@ fn _socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
     unsafe {
         let mut buffer = MaybeUninit::<SocketType>::uninit();
         let mut out_len = size_of::<SocketType>() as linux_raw_sys::general::socklen_t;
-        let slice =
-            std::slice::from_raw_parts_mut(buffer.as_mut_ptr() as *mut u8, size_of::<SocketType>());
+        let slice = std::slice::from_raw_parts_mut(
+            buffer.as_mut_ptr().cast::<u8>(),
+            size_of::<SocketType>(),
+        );
         crate::linux_raw::getsockopt(
             fd,
             linux_raw_sys::general::SOL_SOCKET as i32,
